package UDPTerm;

import java.net.DatagramSocket;
import java.net.InetAddress;
import java.net.UnknownHostException;
import java.net.NetworkInterface;
import java.net.SocketException;
import java.io.InputStreamReader;
import java.io.BufferedReader;
import java.io.IOException;

import UDP.UDPClient;


public class Client {

	public static void main(String[] args) {
        // local variables 
		final int CommandsNum = 8;
		final int IPRequestCmd = 6; 
		String[] CommandStrs = new String[CommandsNum];
		String ResponseStr;
		String IPAddrStr;
		
		BufferedReader cin = new BufferedReader(new InputStreamReader(System.in));

		// list of supported commands 
		CommandStrs[0] = "LED GREEN ON";
		CommandStrs[1] = "LED GREEN OFF";
		CommandStrs[2] = "LED BLUE ON";
		CommandStrs[3] = "LED BLUE OFF";
		CommandStrs[4] = "LED RED ON";
		CommandStrs[5] = "LED RED OFF";
		CommandStrs[6] = "K64FIPRQ";
		CommandStrs[7] = "EXIT";

		// initial general broadcast IP
		String broadcastIP = "255.255.255.255";
		
		try(DatagramSocket s=new DatagramSocket())
		{
		    // local host IP address
			s.connect(InetAddress.getByAddress(new byte[]{1,1,1,1}), 0);
		    byte[] ip_addr_raw = s.getLocalAddress().getAddress();
		    //ip_addr_raw[3] = (byte)255;
		    // local subnet mask
		    InetAddress localHost = InetAddress.getLocalHost();
		    NetworkInterface networkInterface = NetworkInterface.getByInetAddress(localHost);
		    short maskPrefixLength = networkInterface.getInterfaceAddresses().get(0).getNetworkPrefixLength();
		    // create a local subnet broadcast address (inverse)
		    byte[] ip_addr_mask_raw_inverse = new byte[4];
		    for (int i = 0; i<4; i++) {
		    	if (maskPrefixLength >= 8) {
		    		ip_addr_mask_raw_inverse[i] = (byte)0x0;
		    	} else {
		    		ip_addr_mask_raw_inverse[i] = (byte) ((0xFF >> maskPrefixLength) & 0xFF);
		    	}
		    	ip_addr_raw[i] |= ip_addr_mask_raw_inverse[i];
		    	maskPrefixLength -= 8;
		    }
		    // create broadcast IP address as a string
		    broadcastIP = getIpAddress(ip_addr_raw);
		} catch (UnknownHostException e) {
			e.printStackTrace();
		} catch (SocketException e1) {
			e1.printStackTrace();
		}
		// using broadcast IP to find the device
		UDPClient udpClient = new UDPClient(broadcastIP,7);
		ResponseStr = udpClient.send(CommandStrs[IPRequestCmd]);
		// if the response from the FTMD-K64F UDP demo application has been received
		if (ResponseStr != null) {
			if (ResponseStr.startsWith("IP addr: ")) {
				IPAddrStr = ResponseStr.substring(9);
				// bind the client to the received IP address and the port 7
				udpClient = new UDPClient(IPAddrStr,7);
			} else {
				System.out.println("FRDM-K64F device not found");
				return;
			}
		} else {
			System.out.println("FRDM-K64F device not found");
			return;
		}
		
		// print the help for this application
		for (int index = 0; index < CommandsNum; index++) {
			System.out.println(index + ". " + CommandStrs[index]);
		}
		System.out.println("Write exit to finish this application.");
		
	    // command string	 
        String command = new String();
        do {
        	// read line from the console
        	try {
        		command = cin.readLine();
        	} catch (IOException e) {
				e.printStackTrace();
			}
        	//if the command is not the exit command of this application 
            if (!command.equalsIgnoreCase("exit")) {
            	try {
            		// verify the command number 
                	int index = Integer.parseInt(command);
    	        	if ((index >= 0) && (index < CommandsNum)) {
    	        		// send the selected command to the target device (FRDM-K64F) 
    	        		String udpResponse = udpClient.send(CommandStrs[index]);
    	        		if (udpResponse != null) {
    	        			System.out.println(udpResponse);
    	        		} else {
    	        			System.out.println("The device does not reply (disconnected or demo application has finished) ");
    	        		}
    	        	} else {
    	        	    System.out.println("Incorrect command");
    	        	}
            	} catch (java.lang.NumberFormatException e) {
            		System.out.println("Unknown command");
            	}
	        }
        } while (!command.equalsIgnoreCase("exit"));
        return;
	}
	
	// function for conversion of the raw IP address to string
    public static String getIpAddress(byte[] rawBytes) {
        int i = 4;
        String ipAddress = "";
        for (byte raw : rawBytes)
        {
            ipAddress += (raw & 0xFF);
            if (--i > 0)
            {
                ipAddress += ".";
            }
        }
        return ipAddress;
    }

}
